;==============================================================================
; Working Register Interrupt Save Area
;------------------------------------------------------------------------------

; Notes:
;
; When an interrupt routine is entered the code must save the W register before
; it can begin saving any of the other important registers (e.g. STATUS, PCLATH
; and FSR). As switching bank would alter the STATUS file we have two choices
; depending on the PIC device model:
;
; 1. Reserve a shared register file that can be accessed from any bank, or
;
; 2. Reserve one, two or four files which occupy the same relative position in
;    all available banks.
;
; Unfortunately there is no easy way to do this for all PIC versions so we have
; to use conditional compilation for each supported device.

;==============================================================================
; Revision History:
;
; 2005-01-24 AJ Initial version
;------------------------------------------------------------------------------

                include ../pic.inc

                if      HAS_INTERRUPTS

                global  _WREG

                ifdef   __12F675
                udata_shr
_WREG           res     1
                endif

                ifdef   __16F628
                udata_shr
_WREG           res     1
                endif

                ifdef   __16F74
.save0          udata   h'20'
_WREG           res     1

.save1          udata   h'a0'
                res     1
                endif

                endif
                end
